@@ -10,6 +10,9 @@
 /* Procedure main() performs the basic route steps without any	*/
 /* interaction or scripting, writes a DEF file as output, and	*/
 /* exits.							*/
+/*								*/
+/* Precedure mimics the "standard_route" script (up to date as	*/
+/* of November 25, 2015)					*/
 /*--------------------------------------------------------------*/
 
 int
@@ -21,8 +24,12 @@ main(int argc, char *argv[])
     if (result != 0) return result;
 
     read_def(NULL);
-    dofirststage(0);
-    dosecondstage(0);
+    maskMode = MASK_AUTO;
+    dofirststage(0, -1);
+    maskMode = MASK_NONE;
+    result = dosecondstage(0, FALSE);
+    if (result < 5)
+	dosecondstage(0, FALSE);
     write_def(NULL);
     return 0;
 }
