@@ -633,21 +633,51 @@ int set_routes_to_net(NET net, int newflags, POINT *pushlist, SEG bbox,
     return result;
 }
 
+/*--------------------------------------------------------------*/
+/* Used by find_colliding() (see below).  Save net "netnum"	*/
+/* to the list of colliding nets if it is not already in the	*/
+/* list.  Return 1 if the list got longer, 0 otherwise.		*/
+/*--------------------------------------------------------------*/
+
+int
+addcollidingnet(NETLIST *nlptr, int netnum)
+{
+    NETLIST cnl;
+    NET fnet;
+    int i;
+
+    for (cnl = *nlptr; cnl; cnl = cnl->next)
+	if (cnl->net->netnum == netnum)
+	    return 0;
+
+    for (i = 0; i < Numnets; i++) {
+	fnet = Nlnets[i];
+	if (fnet->netnum == netnum) {
+	    cnl = (NETLIST)malloc(sizeof(struct netlist_));
+	    cnl->net = fnet;
+	    cnl->next = *nlptr;
+	    *nlptr = cnl;
+	    return 1;
+	}
+    }
+    return 0;
+}
+
 /*--------------------------------------------------------------*/
 /* Find nets that are colliding with the given net "net", and	*/
 /* create and return a list of them.				*/
 /*--------------------------------------------------------------*/
 
-NETLIST find_colliding(NET net)
+NETLIST find_colliding(NET net, int *ripnum)
 {
    NETLIST nl = (NETLIST)NULL, cnl;
-   NET  fnet;
    ROUTE rt;
    SEG seg;
-   int lay, i, x, y, orignet;
+   int lay, i, x, y, orignet, rnum;
 
    /* Scan the routed points for recorded collisions.	*/
 
+   rnum = 0;
    for (rt = net->routes; rt; rt = rt->next) {
       if (rt->segments) {
 	 for (seg = rt->segments; seg; seg = seg->next) {
@@ -661,30 +691,53 @@ NETLIST find_colliding(NET net)
 	    // belong to a different net.
 
 	    while (1) {
-	       orignet = Obs[lay][OGRID(x, y, lay)] & NETNUM_MASK;
+	       orignet = Obs[lay][OGRID(x, y, lay)] & ROUTED_NET_MASK;
 
-	       if (orignet != net->netnum) {
+	       if (orignet == DRC_BLOCKAGE) {
 
-	          /* Route collision.  Save this net if it is	*/
-	          /* not already in the list of colliding nets.	*/
+		  /* If original position was a DRC-related blockage,	*/
+		  /* find out which net or nets would be in conflict.	*/
 
-	          for (cnl = nl; cnl; cnl = cnl->next) {
-		     if (cnl->net->netnum == orignet)
-		        break;
-	          }
-	          if (cnl == NULL) {
-		     for (i = 0; i < Numnets; i++) {
-			fnet = Nlnets[i];
-		        if (fnet->netnum == orignet) {
-			   cnl = (NETLIST)malloc(sizeof(struct netlist_));
-		           cnl->net = fnet;
-		           cnl->next = nl;
-		           nl = cnl;
-			   break;
+		  if (needblock[lay] & (ROUTEBLOCKX | VIABLOCKX)) {
+		     if (x < NumChannelsX[lay] - 1) {
+		        orignet = Obs[lay][OGRID(x + 1, y, lay)] & ROUTED_NET_MASK;
+		        if (!(orignet & NO_NET)) {
+			   orignet &= NETNUM_MASK;
+			   if ((orignet != 0) && (orignet != net->netnum))
+		 	       rnum += addcollidingnet(&nl, orignet);
+		        }
+		     }
+		     if (x > 0) {
+		        orignet = Obs[lay][OGRID(x - 1, y, lay)] & ROUTED_NET_MASK;
+		        if (!(orignet & NO_NET)) {
+			   orignet &= NETNUM_MASK;
+			   if ((orignet != 0) && (orignet != net->netnum))
+		 	       rnum += addcollidingnet(&nl, orignet);
+		        }
+		     }
+		  }
+		  if (needblock[lay] & (ROUTEBLOCKY | VIABLOCKY)) {
+		     if (y < NumChannelsY[lay] - 1) {
+		        orignet = Obs[lay][OGRID(x, y + 1, lay)] & ROUTED_NET_MASK;
+		        if (!(orignet & NO_NET)) {
+			   orignet &= NETNUM_MASK;
+			   if ((orignet != 0) && (orignet != net->netnum))
+		 	       rnum += addcollidingnet(&nl, orignet);
+			}
+		     }
+		     if (y > 0) {
+		        orignet = Obs[lay][OGRID(x, y - 1, lay)] & ROUTED_NET_MASK;
+		        if (!(orignet & NO_NET)) {
+			   orignet &= NETNUM_MASK;
+			   if ((orignet != 0) && (orignet != net->netnum))
+		 	       rnum += addcollidingnet(&nl, orignet);
 			}
 		     }
 		  }
 	       }
+	       else if ((orignet & NETNUM_MASK) != net->netnum)
+		  rnum += addcollidingnet(&nl, (orignet & NETNUM_MASK));
+
 	       if ((x == seg->x2) && (y == seg->y2)) break;
 
 	       if (x < seg->x2) x++;
@@ -707,6 +760,7 @@ NETLIST find_colliding(NET net)
       Fprintf(stdout, "\n");
    }
 
+   if (ripnum) *ripnum = rnum;
    return nl;
 }
 
@@ -759,28 +813,32 @@ u_char ripup_net(NET net, u_char restore)
 		     else
 		        Obs[lay][OGRID(x, y, lay)] = NO_NET | dir;
 		  }
+	          else {
+		     // Clear routed mask bit
+		     Obs[lay][OGRID(x, y, lay)] &= ~ROUTED_NET;
+		  }
 
 		  // Routes which had blockages added on the sides due
-		  // to spacing constraints have (NO_NET | ROUTED_NET)
-		  // set;  these flags should be removed.
+		  // to spacing constraints have DRC_BLOCKAGE set;
+		  // these flags should be removed.
 
 		  if (needblock[lay] & (ROUTEBLOCKX | VIABLOCKX)) {
 		     if ((x > 0) && ((Obs[lay][OGRID(x - 1, y, lay)] &
-				(NO_NET | ROUTED_NET)) == (NO_NET | ROUTED_NET)))
-			Obs[lay][OGRID(x - 1, y, lay)] &= ~(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE) == DRC_BLOCKAGE))
+			Obs[lay][OGRID(x - 1, y, lay)] &= ~DRC_BLOCKAGE;
 		     else if ((x < NumChannelsX[lay] - 1) &&
 				((Obs[lay][OGRID(x + 1, y, lay)] &
-				(NO_NET | ROUTED_NET)) == (NO_NET | ROUTED_NET)))
-			Obs[lay][OGRID(x + 1, y, lay)] &= ~(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE) == DRC_BLOCKAGE))
+			Obs[lay][OGRID(x + 1, y, lay)] &= ~DRC_BLOCKAGE;
 		  }
 		  if (needblock[lay] & (ROUTEBLOCKY | VIABLOCKY)) {
 		     if ((y > 0) && ((Obs[lay][OGRID(x, y - 1, lay)] &
-				(NO_NET | ROUTED_NET)) == (NO_NET | ROUTED_NET)))
-			Obs[lay][OGRID(x, y - 1, lay)] &= ~(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE) == DRC_BLOCKAGE))
+			Obs[lay][OGRID(x, y - 1, lay)] &= ~DRC_BLOCKAGE;
 		     else if ((y < NumChannelsY[lay] - 1) &&
 				((Obs[lay][OGRID(x, y + 1, lay)] &
-				(NO_NET | ROUTED_NET)) == (NO_NET | ROUTED_NET)))
-			Obs[lay][OGRID(x, y + 1, lay)] &= ~(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE) == DRC_BLOCKAGE))
+			Obs[lay][OGRID(x, y + 1, lay)] &= ~DRC_BLOCKAGE;
 		  }
 	       }
 
@@ -854,6 +912,7 @@ u_char ripup_net(NET net, u_char restore)
 int eval_pt(GRIDP *ept, u_char flags, u_char stage)
 {
     int thiscost = 0;
+    int netnum;
     NODE node;
     NETLIST nl;
     PROUTE *Pr, *Pt;
@@ -886,7 +945,8 @@ int eval_pt(GRIDP *ept, u_char flags, u_char stage)
 
     if (!(Pr->flags & (PR_COST | PR_SOURCE))) {
        // 2nd stage allows routes to cross existing routes
-       if (stage && (Pr->prdata.net < MAXNETNUM)) {
+       netnum = Pr->prdata.net;
+       if (stage && (netnum < MAXNETNUM)) {
 	  if ((newpt.lay < Pinlayers) &&
 		Nodesav[newpt.lay][OGRID(newpt.x, newpt.y, newpt.lay)] != NULL)
 	     return 0;			// But cannot route over terminals!
@@ -894,7 +954,7 @@ int eval_pt(GRIDP *ept, u_char flags, u_char stage)
 	  // Is net k in the "noripup" list?  If so, don't route it */
 
 	  for (nl = CurNet->noripup; nl; nl = nl->next) {
-	     if (nl->net->netnum == Pr->prdata.net)
+	     if (nl->net->netnum == netnum)
 		return 0;
 	  }
 
@@ -907,6 +967,81 @@ int eval_pt(GRIDP *ept, u_char flags, u_char stage)
 	  Pr->prdata.cost = MAXRT;
 	  thiscost = ConflictCost;
        }
+       else if (stage && (netnum == DRC_BLOCKAGE)) {
+	  if ((newpt.lay < Pinlayers) &&
+		Nodesav[newpt.lay][OGRID(newpt.x, newpt.y, newpt.lay)] != NULL)
+	     return 0;			// But cannot route over terminals!
+
+	  // Position does not contain the net number, so we have to
+	  // go looking for it.  Fortunately this is a fairly rare
+	  // occurrance.  But it is necessary to find all neighboring
+	  // nets that might have created the blockage, and refuse to
+	  // route here if any of them are on the noripup list.
+
+	  if (needblock[newpt.lay] & (ROUTEBLOCKX | VIABLOCKX)) {
+	     if (newpt.x < NumChannelsX[newpt.lay] - 1) {
+	        netnum = Obs[newpt.lay][OGRID(newpt.x + 1, newpt.y, newpt.lay)] &
+			ROUTED_NET_MASK;
+	        if (!(netnum & NO_NET)) {
+		   netnum &= NETNUM_MASK;
+		   if ((netnum != 0) && (netnum != CurNet->netnum))
+	              // Is net k in the "noripup" list?  If so, don't route it */
+	              for (nl = CurNet->noripup; nl; nl = nl->next)
+	                 if (nl->net->netnum == netnum)
+		            return 0;
+		}
+	     }
+
+	     if (newpt.x > 0) {
+	        netnum = Obs[newpt.lay][OGRID(newpt.x - 1, newpt.y, newpt.lay)] &
+			ROUTED_NET_MASK;
+	        if (!(netnum & NO_NET)) {
+		   netnum &= NETNUM_MASK;
+		   if ((netnum != 0) && (netnum != CurNet->netnum))
+	              // Is net k in the "noripup" list?  If so, don't route it */
+	              for (nl = CurNet->noripup; nl; nl = nl->next)
+	                 if (nl->net->netnum == netnum)
+		            return 0;
+		}
+	     }
+	  } 
+	  if (needblock[newpt.lay] & (ROUTEBLOCKY | VIABLOCKY)) {
+	     if (newpt.y < NumChannelsY[newpt.lay] - 1) {
+	        netnum = Obs[newpt.lay][OGRID(newpt.x, newpt.y + 1, newpt.lay)] &
+			ROUTED_NET_MASK;
+	        if (!(netnum & NO_NET)) {
+		   netnum &= NETNUM_MASK;
+		   if ((netnum != 0) && (netnum != CurNet->netnum))
+	              // Is net k in the "noripup" list?  If so, don't route it */
+	              for (nl = CurNet->noripup; nl; nl = nl->next)
+	                 if (nl->net->netnum == netnum)
+		            return 0;
+		}
+	     }
+
+	     if (newpt.y > 0) {
+	        netnum = Obs[newpt.lay][OGRID(newpt.x, newpt.y - 1, newpt.lay)] &
+			ROUTED_NET_MASK;
+	        if (!(netnum & NO_NET)) {
+		   netnum &= NETNUM_MASK;
+		   if ((netnum != 0) && (netnum != CurNet->netnum))
+	              // Is net k in the "noripup" list?  If so, don't route it */
+	              for (nl = CurNet->noripup; nl; nl = nl->next)
+	                 if (nl->net->netnum == netnum)
+		            return 0;
+		}
+	     }
+	  }
+
+	  // In case of a collision, we change the grid point to be routable
+	  // but flag it as a point of collision so we can later see what
+	  // were the net numbers of the interfering routes by cross-referencing
+	  // the Obs[][] array.
+
+	  Pr->flags |= (PR_CONFLICT | PR_COST);
+	  Pr->prdata.cost = MAXRT;
+	  thiscost = ConflictCost;
+       }
        else
           return 0;		// Position is not routeable
     }
@@ -996,7 +1131,7 @@ int eval_pt(GRIDP *ept, u_char flags, u_char stage)
 /* adjacent positions in Obs against the mask		*/
 /* NETNUM_MASK, because NETNUM_MASK includes NO_NET.	*/
 /* By replacing only empty and routable positions with	*/
-/* the unique flag combination	NO_NET | ROUTED_NET, it	*/
+/* the unique flag combination	DRC_BLOCKAGE, it	*/
 /* is possible to detect and remove the same if that	*/
 /* net is ripped up, without touching any position	*/
 /* originally marked NO_NET.				*/
@@ -1022,81 +1157,24 @@ void writeback_segment(SEG seg, int netnum)
 		(Obs[seg->layer + 1][OGRID(seg->x1 + 1, seg->y1, seg->layer + 1)]
 		& NETNUM_MASK) == 0)
  	    Obs[seg->layer + 1][OGRID(seg->x1 + 1, seg->y1, seg->layer + 1)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	 if ((seg->x1 > 0) &&
 		(Obs[seg->layer + 1][OGRID(seg->x1 - 1, seg->y1, seg->layer + 1)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer + 1][OGRID(seg->x1 - 1, seg->y1, seg->layer + 1)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
       }
       if (needblock[seg->layer + 1] & VIABLOCKY) {
 	 if ((seg->y1 < (NumChannelsY[seg->layer + 1] - 1)) &&
 		(Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 + 1, seg->layer + 1)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 + 1, seg->layer + 1)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	 if ((seg->y1 > 0) &&
 		(Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 - 1, seg->layer + 1)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 - 1, seg->layer + 1)] =
-			(NO_NET | ROUTED_NET);
-      }
-
-      // Check position on each side for an offset tap on a different net, and
-      // mark the position unroutable.
-      //
-      // NOTE:  This is a bit conservative, as it will block positions next to
-      // an offset tap without checking if the offset distance is enough to
-      // cause a DRC error.  Could be refined. . .
-
-      layer = (seg->layer == 0) ? 0 : seg->layer - 1;
-      if (seg->x1 < (NumChannelsX[layer] - 1)) {
-	 sobs = Obs[layer][OGRID(seg->x1 + 1, seg->y1, layer)];
-	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
-	    if (sobs & STUBROUTE_EW) {
-	       dist = Stub[layer][OGRID(seg->x1 + 1, seg->y1, layer)];
-	       if (dist > 0) {
-		  Obs[layer][OGRID(seg->x1 + 1, seg->y1, layer)] |=
-			(NO_NET | ROUTED_NET);
-	       }
-	    }
-	 }
-      }
-      if (seg->x1 > 0) {
-	 sobs = Obs[layer][OGRID(seg->x1 - 1, seg->y1, layer)];
-	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
-	    if (sobs & STUBROUTE_EW) {
-	       dist = Stub[layer][OGRID(seg->x1 - 1, seg->y1, layer)];
-	       if (dist < 0) {
-		  Obs[layer][OGRID(seg->x1 - 1, seg->y1, layer)] |=
-			(NO_NET | ROUTED_NET);
-	       }
-	    }
-	 }
-      }
-      if (seg->y1 < (NumChannelsY[layer] - 1)) {
-	 sobs = Obs[layer][OGRID(seg->x1, seg->y1 + 1, layer)];
-	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
-	    if (sobs & STUBROUTE_NS) {
-	       dist = Stub[layer][OGRID(seg->x1, seg->y1 + 1, layer)];
-	       if (dist > 0) {
-		  Obs[layer][OGRID(seg->x1, seg->y1 + 1, layer)] |=
-			(NO_NET | ROUTED_NET);
-	       }
-	    }
-	 }
-      }
-      if (seg->y1 > 0) {
-	 sobs = Obs[layer][OGRID(seg->x1, seg->y1 - 1, layer)];
-	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
-	    if (sobs & STUBROUTE_NS) {
-	       dist = Stub[layer][OGRID(seg->x1, seg->y1 - 1, layer)];
-	       if (dist < 0) {
-		  Obs[layer][OGRID(seg->x1, seg->y1 - 1, layer)] |=
-			(NO_NET | ROUTED_NET);
-	       }
-	    }
-	 }
+			DRC_BLOCKAGE;
       }
 
       // If position itself is an offset route, then make the route position
@@ -1104,35 +1182,36 @@ void writeback_segment(SEG seg, int netnum)
       // (Like the above code, there is no check for whether the offset
       // distance is enough to cause a DRC violation.)
 
+      layer = (seg->layer == 0) ? 0 : seg->layer - 1;
       sobs = Obs[seg->layer][OGRID(seg->x1, seg->y1, seg->layer)];
       if (sobs & OFFSET_TAP) {
 	 dist = Stub[layer][OGRID(seg->x1, seg->y1, seg->layer)];
 	 if (sobs & STUBROUTE_EW) {
 	    if ((dist > 0) && (seg->x1 < (NumChannelsX[seg->layer] - 1))) {
 	       Obs[seg->layer][OGRID(seg->x1 + 1, seg->y1, seg->layer)] |=
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	       Obs[seg->layer + 1][OGRID(seg->x1 + 1, seg->y1, seg->layer + 1)] |=
-				(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE;
 	    }
 	    if ((dist < 0) && (seg->x1 > 0)) {
 	       Obs[seg->layer][OGRID(seg->x1 - 1, seg->y1, seg->layer)] |=
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	       Obs[seg->layer + 1][OGRID(seg->x1 - 1, seg->y1, seg->layer + 1)] |=
-				(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE;
 	    }
 	 }
 	 else if (sobs & STUBROUTE_NS) {
 	    if ((dist > 0) && (seg->y1 < (NumChannelsY[seg->layer] - 1))) {
 	       Obs[seg->layer][OGRID(seg->x1, seg->y1 + 1, seg->layer)] |=
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	       Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 + 1, seg->layer + 1)] |=
-				(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE;
 	    }
 	    if ((dist < 0) && (seg->y1 > 0)) {
 	       Obs[seg->layer][OGRID(seg->x1, seg->y1 - 1, seg->layer)] |=
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	       Obs[seg->layer + 1][OGRID(seg->x1, seg->y1 - 1, seg->layer + 1)] |=
-				(NO_NET | ROUTED_NET);
+				DRC_BLOCKAGE;
 	    }
 	 }
       }
@@ -1145,12 +1224,45 @@ void writeback_segment(SEG seg, int netnum)
 		(Obs[seg->layer][OGRID(i, seg->y1 + 1, seg->layer)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer][OGRID(i, seg->y1 + 1, seg->layer)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	 if ((seg->y1 > 0) &&
 		(Obs[seg->layer][OGRID(i, seg->y1 - 1, seg->layer)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer][OGRID(i, seg->y1 - 1, seg->layer)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
+      }
+
+      // Check position on each side for an offset tap on a different net, and
+      // mark the position unroutable.
+      //
+      // NOTE:  This is a bit conservative, as it will block positions next to
+      // an offset tap without checking if the offset distance is enough to
+      // cause a DRC error.  Could be refined. . .
+
+      layer = (seg->layer == 0) ? 0 : seg->layer - 1;
+      if (seg->y1 < (NumChannelsY[layer] - 1)) {
+	 sobs = Obs[layer][OGRID(i, seg->y1 + 1, layer)];
+	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
+	    if (sobs & STUBROUTE_NS) {
+	       dist = Stub[layer][OGRID(i, seg->y1 + 1, layer)];
+	       if (dist < 0) {
+		  Obs[layer][OGRID(i, seg->y1 + 1, layer)] |=
+			DRC_BLOCKAGE;
+	       }
+	    }
+	 }
+      }
+      if (seg->y1 > 0) {
+	 sobs = Obs[layer][OGRID(i, seg->y1 - 1, layer)];
+	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
+	    if (sobs & STUBROUTE_NS) {
+	       dist = Stub[layer][OGRID(i, seg->y1 - 1, layer)];
+	       if (dist > 0) {
+		  Obs[layer][OGRID(i, seg->y1 - 1, layer)] |=
+			DRC_BLOCKAGE;
+	       }
+	    }
+	 }
       }
       if (i == seg->x2) break;
    }
@@ -1161,12 +1273,41 @@ void writeback_segment(SEG seg, int netnum)
 		(Obs[seg->layer][OGRID(seg->x1 + 1, i, seg->layer)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer][OGRID(seg->x1 + 1, i, seg->layer)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
 	 if ((seg->x1 > 0) &&
 		(Obs[seg->layer][OGRID(seg->x1 - 1, i, seg->layer)]
 		& NETNUM_MASK) == 0)
 	    Obs[seg->layer][OGRID(seg->x1 - 1, i, seg->layer)] =
-			(NO_NET | ROUTED_NET);
+			DRC_BLOCKAGE;
+      }
+
+      // Check position on each side for an offset tap on a different net, and
+      // mark the position unroutable (see above).
+
+      layer = (seg->layer == 0) ? 0 : seg->layer - 1;
+      if (seg->x1 < (NumChannelsX[layer] - 1)) {
+	 sobs = Obs[layer][OGRID(seg->x1 + 1, i, layer)];
+	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
+	    if (sobs & STUBROUTE_EW) {
+	       dist = Stub[layer][OGRID(seg->x1 + 1, i, layer)];
+	       if (dist < 0) {
+		  Obs[layer][OGRID(seg->x1 + 1, i, layer)] |=
+			DRC_BLOCKAGE;
+	       }
+	    }
+	 }
+      }
+      if (seg->x1 > 0) {
+	 sobs = Obs[layer][OGRID(seg->x1 - 1, i, layer)];
+	 if ((sobs & OFFSET_TAP) && !(sobs & ROUTED_NET)) {
+	    if (sobs & STUBROUTE_EW) {
+	       dist = Stub[layer][OGRID(seg->x1 - 1, i, layer)];
+	       if (dist > 0) {
+		  Obs[layer][OGRID(seg->x1 - 1, i, layer)] |=
+			DRC_BLOCKAGE;
+	       }
+	    }
+	 }
       }
       if (i == seg->y2) break;
    }
@@ -1737,13 +1878,19 @@ int commit_proute(ROUTE rt, GRIDP *ept, u_char stage)
 
       netnum |= ROUTED_NET;
 
+      // Write back segment, but not on stage 2 or else the
+      // collision information will be lost.  Stage 2 uses
+      // writeback_route to call writeback_segment after the
+      // colliding nets have been ripped up.
+
+      if (stage == (u_char)0)
+	 writeback_segment(seg, netnum);
+
       // If Obs shows this position as an obstruction, then this was a port with
       // no taps in reach of a grid point.  This will be dealt with by moving
       // the via off-grid and onto the port position in emit_routes().
 
       if (stage == (u_char)0) {
-	 writeback_segment(seg, netnum);
-
          if (first && dir1) {
 	    first = (u_char)0;
          }
