@@ -95,7 +95,7 @@ struct gridp_ {
 typedef struct proute_ PROUTE;
 
 struct proute_ {        // partial route
-   u_char flags; 	// values PR_PROCESSED and PR_CONFLICT
+   u_char flags; 	// values PR_PROCESSED and PR_CONFLICT, and others
    union {
       u_int cost;	// cost of route coming from predecessor
       u_int net;	// net number at route point
@@ -135,6 +135,7 @@ struct string_ {
 #define	 ST_VIA			0x02
 #define	 ST_OFFSET_START	0x04	/* (x1, y1) is offset from grid */
 #define	 ST_OFFSET_END		0x08	/* (x2, y2) is offset from grid */
+#define  ST_SPECIAL		0x10	/* wide metal (special net)	*/
 
 typedef struct seg_ *SEG;
 
@@ -202,6 +203,7 @@ struct node_ {
   DPOINT  taps;			// point position for node taps
   DPOINT  extend;		// point position within halo of the tap
   char    *netname;   		// name of net this node belongs to
+  unsigned char numtaps;	// number of actual reachable taps
   int     netnum;               // number of net this node belongs to
   int     numnodes;		// number of nodes on this net
   int	  branchx;		// position of the node branch in x
@@ -294,7 +296,7 @@ struct routeinfo_ {
 // The Stub[] vector indicates the distance needed to avoid the obstruction.
 //
 // The maximum number of nets must not overrun the area used by flags, so
-// the maximum number of nets is 0x7fffff, or 8,388,607 nets
+// the maximum number of nets is 0x1fffff, or 2,097,151 nets
 
 #define PINOBSTRUCTMASK	((u_int)0xe0000000)  // takes values from below
 #define STUBROUTE_NS	((u_int)0x20000000)  // route north or south to reach terminal
@@ -306,13 +308,18 @@ struct routeinfo_ {
 #define BLOCKED_S	((u_int)0x04000000)  // grid point cannot be routed from the S
 #define BLOCKED_E	((u_int)0x02000000)  // grid point cannot be routed from the E
 #define BLOCKED_W	((u_int)0x01000000)  // grid point cannot be routed from the W
-#define BLOCKED_MASK	((u_int)0x0f000000)
-#define ROUTED_NET	((u_int)0x00800000)  // indicates position occupied by a routed
+#define BLOCKED_U	((u_int)0x00800000)  // grid point cannot be routed from top
+#define BLOCKED_D	((u_int)0x00400000)  // grid point cannot be routed from bottom
+#define BLOCKED_MASK	((u_int)0x0fc00000)
+#define ROUTED_NET	((u_int)0x00200000)  // indicates position occupied by a routed
 					     // net
-#define NETNUM_MASK	((u_int)0x107fffff)  // Mask for the net number field
-					     // (includes NO_NET)
+#define MAX_NETNUMS	((u_int)0x001fffff)  // Maximum net number
 
-#define MAX_NETNUMS	((u_int)0x007fffff)  // Maximum net number
+#define NETNUM_MASK	((u_int)0x101fffff)  // Mask for the net number field
+					     // (includes NO_NET)
+#define ROUTED_NET_MASK ((u_int)0x103fffff)  // Mask for the net number field
+					     // (includes NO_NET and ROUTED_NET)
+#define DRC_BLOCKAGE	(NO_NET | ROUTED_NET) // Special case
 
 // Definitions used along with the NO_NET bit.
 #define OBSTRUCT_MASK	((u_int)0x0000000f)  // Tells where obstruction is
@@ -386,6 +393,7 @@ extern u_char keepTrying;
 extern u_char forceRoutable;
 extern u_char maskMode;
 extern u_char mapType;
+extern u_char ripLimit;
 
 extern char *vddnet;
 extern char *gndnet;
@@ -407,9 +415,11 @@ extern char *gndnet;
 extern int    set_num_channels();
 extern int    allocate_obs_array();
 
+void   check_variable_pitch(int, int *, int *);
+
 NET    getnettoroute();
-int    dofirststage(u_char graphdebug);
-int    dosecondstage(u_char graphdebug);
+int    dofirststage(u_char graphdebug, int debug_netnum);
+int    dosecondstage(u_char graphdebug, u_char singlestep);
 int    route_net_ripup(NET net, u_char graphdebug);
 
 void   read_lef(char *filename);
@@ -430,7 +440,7 @@ int    set_route_to_net(NET net, ROUTE rt, int newflags, POINT *pushlist,
 
 void   initMask();
 void   createMask(NET net, u_char slack, u_char halo);
-void   fillMask();
+void   fillMask(u_char value);
 
 void   pathstart(FILE *cmd, int layer, int x, int y, u_char special, double oscale,
 			double invscale, u_char horizontal);
@@ -451,6 +461,7 @@ void   helpmessage();
 void   print_nets(char *filename);
 void   print_routes(char *filename);
 void   print_nlgates(char *filename);
+char  *print_node_name(NODE node);
 
 GATE   DefFindInstance(char *name);
 void   DefHashInstance(GATE gateginfo);
