@@ -28,6 +28,8 @@ Tcl_HashTable QrouterTagTable;
 Tcl_Interp *qrouterinterp;
 Tcl_Interp *consoleinterp;
 
+int stepnet = -1;
+
 /* This hash table speeds up DEF file reading */
 
 Tcl_HashTable InstanceTable;
@@ -704,6 +706,7 @@ NET LookupNet(char *netname)
 /*			real-time.  This slows down the	*/
 /*			algorithm and is intended only	*/
 /*			for diagnostic use.		*/
+/*  stage1 step		Single-step stage one.		*/
 /*  stage1 mask none	Don't limit the search area	*/
 /*  stage1 mask auto	Select the mask automatically	*/
 /*  stage1 mask bbox	Use the net bbox as a mask	*/
@@ -717,15 +720,16 @@ NET LookupNet(char *netname)
 int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 	int objc, Tcl_Obj *CONST objv[])
 {
-    u_char dodebug = FALSE;
+    u_char dodebug;
+    u_char dostep;
     int i, idx, idx2, val, result, failcount;
     NET net = NULL;
 
     static char *subCmds[] = {
-	"debug", "mask", "route", "force", NULL
+	"debug", "mask", "route", "force", "step", NULL
     };
     enum SubIdx {
-	DebugIdx, MaskIdx, RouteIdx, ForceIdx
+	DebugIdx, MaskIdx, RouteIdx, ForceIdx, StepIdx
     };
    
     static char *maskSubCmds[] = {
@@ -735,7 +739,13 @@ int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 	NoneIdx, AutoIdx, BboxIdx
     };
 
+    // Command defaults
+
+    dodebug = FALSE;
+    dostep = FALSE;
+    maskMode = MASK_AUTO;	// Mask mode is auto unless specified
     forceRoutable = FALSE;	// Don't force unless specified
+
     if (objc >= 2) {
 	for (i = 1; i < objc; i++) {
 
@@ -749,6 +759,10 @@ int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 		    dodebug = TRUE;
 		    break;
 
+		case StepIdx:
+		    dostep = TRUE;
+		    break;
+
 		case ForceIdx:
 		    forceRoutable = TRUE;
 		    break;
@@ -802,8 +816,11 @@ int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 	}
     }
 
+    if (dostep == FALSE) stepnet = -1;
+    else stepnet++;
+
     if (net == NULL)
-	failcount = dofirststage(dodebug);
+	failcount = dofirststage(dodebug, stepnet);
     else {
 	if ((net != NULL) && (net->netnodes != NULL)) {
 	    result = doroute(net, (u_char)0, dodebug);
@@ -812,6 +829,8 @@ int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
     }
     Tcl_SetObjResult(interp, Tcl_NewIntObj(failcount));
 
+    if (stepnet >= (Numnets - 1)) stepnet = -1;
+
     return QrouterTagCallback(interp, objc, objv);
 }
 
@@ -832,28 +851,33 @@ int qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 /*			real-time.  This slows down the	*/
 /*			algorithm and is intended only	*/
 /*			for diagnostic use.		*/
+/*  stage2 step		Single-step stage two		*/
 /*  stage2 mask none	Don't limit the search area	*/
 /*  stage2 mask auto	Select the mask automatically	*/
 /*  stage2 mask bbox	Use the net bbox as a mask	*/
 /*  stage2 mask <value> Set the mask size to <value>,	*/
 /*			an integer typ. 0 and up.	*/
+/*  stage2 limit <n>	Fail route if solution collides	*/
+/*			with more than <n> nets.	*/
 /*  stage2 route <net>	Route net named <net> only.	*/
 /*							*/
 /*  stage2 force	Force a terminal to be routable	*/
+/*  stage2 tries <n>	Keep trying n additional times	*/
 /*------------------------------------------------------*/
 
 int qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 	int objc, Tcl_Obj *CONST objv[])
 {
-    u_char dodebug = FALSE;
+    u_char dodebug;
+    u_char dostep;
     int i, idx, idx2, val, result, failcount;
     NET net = NULL;
 
     static char *subCmds[] = {
-	"debug", "mask", "route", "force", NULL
+	"debug", "mask", "limit", "route", "force", "tries", "step", NULL
     };
     enum SubIdx {
-	DebugIdx, MaskIdx, RouteIdx, ForceIdx
+	DebugIdx, MaskIdx, LimitIdx, RouteIdx, ForceIdx, TriesIdx, StepIdx
     };
    
     static char *maskSubCmds[] = {
@@ -863,7 +887,14 @@ int qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 	NoneIdx, AutoIdx, BboxIdx
     };
 
+    // Command defaults
+
+    dodebug = FALSE;
+    dostep = FALSE;
+    maskMode = MASK_AUTO;	// Mask mode is auto unless specified
     forceRoutable = FALSE;	// Don't force unless specified
+    ripLimit = 10;		// Rip limit is 10 unless specified
+
     if (objc >= 2) {
 	for (i = 1; i < objc; i++) {
 
@@ -876,10 +907,25 @@ int qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 		case DebugIdx:
 		    dodebug = TRUE;
 		    break;
+
+		case StepIdx:
+		    dostep = TRUE;
+		    break;
 	
 		case ForceIdx:
 		    forceRoutable = TRUE;
 		    break;
+
+		case TriesIdx:
+		    if (i >= objc - 1) {
+			Tcl_WrongNumArgs(interp, 0, objv, "tries ?num?");
+			return TCL_ERROR;
+		    }
+		    i++;
+		    result = Tcl_GetIntFromObj(interp, objv[i], &val);
+		    if (result != TCL_OK) return result;
+		    keepTrying = (u_char)val;
+		    break;
 	
 		case RouteIdx:
 		    if (i >= objc - 1) {
@@ -894,6 +940,17 @@ int qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 		    }
 		    break;
 
+		case LimitIdx:
+		    if (i >= objc - 1) {
+			Tcl_WrongNumArgs(interp, 0, objv, "limit ?num?");
+			return TCL_ERROR;
+		    }
+		    i++;
+		    result = Tcl_GetIntFromObj(interp, objv[i], &val);
+		    if (result != TCL_OK) return result;
+		    ripLimit = (u_char)val;
+		    break;
+	
 		case MaskIdx:
 		    if (i >= objc - 1) {
 			Tcl_WrongNumArgs(interp, 0, objv, "mask ?type?");
@@ -931,7 +988,7 @@ int qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
     }
 
     if (net == NULL)
-	failcount = dosecondstage(dodebug);
+	failcount = dosecondstage(dodebug, dostep);
     else
 	failcount = route_net_ripup(net, dodebug);
     Tcl_SetObjResult(interp, Tcl_NewIntObj(failcount));
@@ -1174,7 +1231,7 @@ int qrouter_readconfig(ClientData clientData, Tcl_Interp *interp,
 		NULL);
 	return TCL_ERROR;
     }
-    read_config(configFILE);
+    read_config(configFILE, FALSE);
     return QrouterTagCallback(interp, objc, objv);
 }
 
@@ -1329,7 +1386,7 @@ int qrouter_priority(ClientData clientData, Tcl_Interp *interp,
 	Tcl_SetObjResult(interp, lobj);
     }
     else {
-	for (i = objc - 1; i > 0; i++) {
+	for (i = objc - 1; i > 0; i--) {
 	    netname = Tcl_GetString(objv[i]);
 	    net = LookupNet(netname);
 	    if (net == NULL) {
